@@ -16,13 +16,15 @@ import math
 import numbers
 import os
 import re
-from translator.toscalib.tosca_template import ToscaTemplate
-from translator.toscalib.utils.gettextutils import _
-import translator.toscalib.utils.yamlparser
+from six.moves.urllib.parse import urlparse
 import yaml
 
-YAML_ORDER_PARSER = translator.toscalib.utils.yamlparser.simple_ordered_parse
+from toscaparser.utils.gettextutils import _
+import toscaparser.utils.yamlparser
+
+YAML_ORDER_PARSER = toscaparser.utils.yamlparser.simple_ordered_parse
 log = logging.getLogger('tosca')
+log = logging.getLogger('heat-translator')
 
 
 class MemoryUnit(object):
@@ -74,6 +76,7 @@ class MemoryUnit(object):
 
             msg = _('Provided unit "{0}" is not valid. The valid units are'
                     ' {1}').format(unit, MemoryUnit.UNIT_SIZE_DICT.keys())
+            log.warning(msg)
             raise ValueError(msg)
 
 
@@ -95,10 +98,13 @@ class CompareUtils(object):
         both_equal = True
         for dict1_item, dict2_item in zip(dict1.items(), dict2.items()):
             if dict1_item != dict2_item:
-                log.warning(CompareUtils.MISMATCH_VALUE2_LABEL,
-                            ": %s \n is not equal to \n",
-                            CompareUtils.MISMATCH_VALUE1_LABEL,
-                            ": %s", dict1_item, dict2_item)
+                msg = (_("%(label1)s: %(item1)s \n is not equal to \n:"
+                         "%(label2)s: %(item2)s")
+                       % {'label1': CompareUtils.MISMATCH_VALUE2_LABEL,
+                          'item1': dict1_item,
+                          'label2': CompareUtils.MISMATCH_VALUE1_LABEL,
+                          'item2': dict2_item})
+                log.warning(msg)
                 both_equal = False
                 break
         return both_equal
@@ -207,7 +213,7 @@ class TranslationUtils(object):
         '''Verify tosca translation against the given hot specification.
 
         inputs:
-        tosca_file: relative path to tosca input
+        tosca_file: relative local path or URL to the tosca input file
         hot_file: relative path to expected hot output
         params: dictionary of parameter name value pairs
 
@@ -215,20 +221,41 @@ class TranslationUtils(object):
         of the given tosca_file and the given hot_file.
         '''
 
+        from toscaparser.tosca_template import ToscaTemplate
+        from translator.hot.tosca_translator import TOSCATranslator
+
         tosca_tpl = os.path.join(
             os.path.dirname(os.path.abspath(__file__)), tosca_file)
+        a_file = os.path.isfile(tosca_tpl)
+        if not a_file:
+            tosca_tpl = tosca_file
+
         expected_hot_tpl = os.path.join(
             os.path.dirname(os.path.abspath(__file__)), hot_file)
 
-        tosca = ToscaTemplate(tosca_tpl, params)
-        translate = translator.hot.tosca_translator.TOSCATranslator(tosca,
-                                                                    params)
+        tosca = ToscaTemplate(tosca_tpl, params, a_file)
+        translate = TOSCATranslator(tosca, params)
+
         output = translate.translate()
-        output_dict = translator.toscalib.utils.yamlparser.simple_parse(output)
+        output_dict = toscaparser.utils.yamlparser.simple_parse(output)
         expected_output_dict = YamlUtils.get_dict(expected_hot_tpl)
         return CompareUtils.diff_dicts(output_dict, expected_output_dict)
 
 
+class UrlUtils(object):
+
+    @staticmethod
+    def validate_url(path):
+        """Validates whether the given path is a URL or not.
+
+        If the given path includes a scheme (http, https, ftp, ...) and a net
+        location (a domain name such as www.github.com) it is validated as a
+        URL.
+        """
+        parsed = urlparse(path)
+        return bool(parsed.scheme) and bool(parsed.netloc)
+
+
 def str_to_num(value):
     """Convert a string representation of a number into a numeric type."""
     if isinstance(value, numbers.Number):
